package centrifuge

import (
	"github.com/centrifugal/centrifuge/internal/proto"
)

// Define some aliases to internal protocol types generated by gogoprotobuf.
// This allows us to simply expose only subset of useful types to centrifuge
// package keeping types that is not supposed to be in public API hidden from
// library users.
type (
	// Error represents client reply error.
	Error = proto.Error
	// Raw represents raw bytes.
	Raw = proto.Raw
	// Publication allows to deliver custom payload to all channel subscribers.
	Publication = proto.Publication
	// Join sent to channel after someone subscribed.
	Join = proto.Join
	// Leave sent to channel after someone unsubscribed.
	Leave = proto.Leave
	// ClientInfo is short information about client connection.
	ClientInfo = proto.ClientInfo
	// ProtocolType represents client connection transport encoding format.
	ProtocolType = proto.ProtocolType
	// EncodingType represents client payload encoding format.
	EncodingType = proto.EncodingType
	// Push wraps Publication, Join or Leave.
	Push = proto.Push
)

// Push types.
const (
	PushTypePublication = proto.PushTypePublication
	PushTypeJoin        = proto.PushTypeJoin
	PushTypeLeave       = proto.PushTypeLeave
)

// Protocol types.
const (
	ProtocolTypeJSON     = proto.ProtocolTypeJSON
	ProtocolTypeProtobuf = proto.ProtocolTypeProtobuf
)

// Encoding types.
const (
	EncodingTypeJSON   = proto.EncodingTypeJSON
	EncodingTypeBinary = proto.EncodingTypeBinary
)
